/**
 * File:
 *  anim.h
 *
 * Notes:
 *  This file contains the prototype of the animation object.  This can be used
 * to create a animation sequence.
 *
 * History:
 *  07/05/2012 Created by Simon Platten
 *  13/05/2012 Added createSchedule
 */
#ifndef ANIM_H
  #define ANIM_H

  #include "fader.h"

  typedef struct {
    byte     bPin;                     // The pin to send to the port
    uint     uintPreDelay;             // The count to wait before applying
    uint     uintPostDelay;            // The count to wait after applying
    faderDef fader;                    // see fader.h
  } outputScheduleDef;

  typedef struct _animDef {
    volatile ushort*   pPort;          // The port the output is associated with
    byte               bTotalSteps;    // The number of steps in the animation
    union {
      char bits;
      struct {
          unsigned uLoop   : 1;        // Flag to loop
          unsigned uBounce : 1;        // Flag to indicate bounce required
          unsigned uDir    : 1;        // Flag to indicate direction 0=Up, 1=Dwn
          unsigned uStop   : 1;        // Flag to indicate animation complete
      };
    } flags;
    outputScheduleDef* parySeq;        // The sequence associated with this anim
    int                intIndex;       // Place in sequence
    byte               bStage;         // Stage in current animation
    uint               uintCtr;        // Current delay value
    struct _animDef*   pCurrent;       // Link to current animation from links
    struct _animDef*   pHead;          // Link to first node in animation
    struct _animDef*   pTail;          // Link to last node in animation
    struct _animDef*   pPrev;          // Link to previous node in animation
    struct _animDef*   pNext;          // Link to next node in animation
  } animDef;
  
// Prototypes
  void initAnim(animDef* pAnim, int intSteps);
  void joinAnims(animDef* pDst, animDef* pSrc);
  void playAnimation(animDef* pAnim);
#endif