/**
 * File:
 *  fader.h
 *
 * Notes:
 *  This file contains the prototype of the fader object.  This can be used
 * to create a fade in or fade out effect on an output pin.
 *
 * History:
 *  07/05/2012 Created by Simon Platten
 *  13/05/2012 Added createSchedule
 */
#ifndef FADER_H
  #define FADER_H

#ifndef ulong
  typedef unsigned long ulong;
#endif
#ifndef uint
  typedef unsigned int uint;
#endif
#ifndef ushort
  typedef unsigned short ushort;
#endif
#ifndef byte
  typedef unsigned char byte;
#endif
#ifndef boolean
  typedef enum ebool { FALSE, TRUE } boolean;
#endif
#ifndef NULL
  #define NULL          ((void*)0)
#endif

  typedef struct {
// 0 = On, 1 = Off
    uint aryuintStates[2];
    uint uintRepeat;
  } scheduleDef;

  typedef struct faderNode {
    unsigned uBounce    : 1;   // 0 = Off, 1 = Bounce up and down thru sch.
    unsigned uDirection : 1;   // 0 = Up,  1 = Down
    unsigned uLoop      : 1;   // 0 = Off, 1 = Loop continuously
    unsigned uPin       : 3;   // The pin number 0 to 7
    unsigned uState     : 1;   // 0 = On,  1 = Off
    unsigned uStop      : 1;   // 0 = Run, 1 = Stop
    volatile ushort*  pPort;   // The port the output is associated with
    uint     uintSteps;        // The number of steps in the schedule
    int      intStep;          // The current step in the schedule
    int      intLastStep;      // The last step performed
    uint     uintCtr;          // The current count of the action in the sch.
    uint     uintRepeat;       // The number of times to repeat the step
// Only applies when loop is off
    void     (*pCB)();         // Call-back to call when complete
// Schedule references
    scheduleDef* pSchItem;     // The current item in the schedule
// The next node in the linked list
    struct faderNode* pNext;
  } faderDef;

// Prototypes
  boolean createFader(faderDef* pFader, byte* pPort, byte uPin,
                      uint uintSteps, boolean blnDir, boolean blnBounce,  
                      boolean blnLoop, boolean blnState, boolean blnStopped,
                      void (*pCB)());
  int     createSchedule(uint uintInc, uint uintRepeat);
  void    serviceFaders();
#endif